@@ -5,6 +5,7 @@
 from __future__ import absolute_import, division
 
 import logging
+import warnings
 import os
 from os.path import join as pjoin
 import signal
@@ -35,6 +36,7 @@ psgrn_gravity_names = ('gd', 'gr')
 psgrn_components = 'ep ss ds cl'.split()
 
 km = 1000.
+day = 3600. * 24
 
 guts_prefix = 'pf'
 logger = logging.getLogger('pyrocko.fomosto.psgrn_pscmp')
@@ -80,6 +82,14 @@ def cake_model_to_config(mod):
 
 
 class PsGrnSpatialSampling(Object):
+
+    '''
+    Definition of spatial sampling for PSGRN.
+
+    Note: attributes in this class use non-standard units [km] to be consistent
+    with PSGRN text file input. Please read the documentation carefully.
+    '''
+
     n_steps = Int.T(default=10)
     start_distance = Float.T(default=0.)    # start sampling [km] from source
     end_distance = Float.T(default=100.)    # end
@@ -91,27 +101,39 @@ class PsGrnSpatialSampling(Object):
 
 class PsGrnConfig(Object):
 
+    '''
+    Configuration for PSGRN.
+
+    Note: attributes in this class use non-standard units [km] and [days] to
+    be consistent with PSGRN text file input. Please read the documentation
+    carefully.
+    '''
+
     version = String.T(default='2008a')
 
     sampling_interval = Float.T(
         default=1.0,
-        help='exponential sampling 1.- equidistant, > 1. decreasing sampling'
+        help='Exponential sampling 1.- equidistant, > 1. decreasing sampling'
              ' with distance')
     n_time_samples = Int.T(
-        default=1,
+        optional=True,
         help='Number of temporal GF samples up to max_time. Has to be equal'
              ' to a power of 2! If not, next power of 2 is taken.')
     max_time = Float.T(
-        default=1.,
+        optional=True,
         help='Maximum time [days] after seismic event.')
 
     gf_depth_spacing = Float.T(
-        default=1.,
-        help='spatial depth spacing [km] for the medium properties.')
+        optional=True,
+        help='Depth spacing [km] for the observation points. '
+             'If not defined depth spacing from the `StoreConfig` is used')
     gf_distance_spacing = Float.T(
-        default=10.,
-        help='spatial distance spacing [km] for the medium properties.')
-    observation_depth = Float.T(default=0.)
+        optional=True,
+        help='Spatial spacing [km] for the observation points. '
+             'If not defined distance spacing from the `StoreConfig` is used')
+    observation_depth = Float.T(
+        default=0.,
+        help='Depth of observation points [km]')
 
     def items(self):
         return dict(self.T.inamevals(self))
@@ -626,20 +648,50 @@ MTDev = {
     }
 
 
+def get_nullification_factor(mu, lame_lambda):
+    '''
+    Factor that needs to be multiplied to 2 of the tensile sources to
+    eliminate two of the isotropic components.
+    '''
+    return -lame_lambda / (2. * mu + 2. * lame_lambda)
+
+
+def get_trace_normalization_factor(mu, lame_lambda, nullification):
+    '''
+    Factor to be multiplied to elementary GF trace to have unit displacement.
+    '''
+    return 1. / ((2. * mu) + lame_lambda + (2. * lame_lambda * nullification))
+
+
+def get_iso_scaling_factors(mu, lame_lambda):
+    nullf = get_nullification_factor(mu, lame_lambda)
+    scale = get_trace_normalization_factor(mu, lame_lambda, nullf)
+    return nullf, scale
+
+
 class PsCmpTensileSF(Location, gf.seismosizer.Cloneable):
     '''
     Compound dislocation of 3 perpendicular, rectangular sources to approximate
     an opening single force couple. NED coordinate system!
+
+    Warning: Mixed standard [m] / non-standard [days] units are used in this
+    class. Please read the documentation carefully.
     '''
 
-    length = Float.T(default=1.0 * km)
-    width = Float.T(default=1.0 * km)
-    torigin = Float.T(default=0.0)
-    idx = String.T(default='nn',
-                   help='Axis Index for opening direction; "nn, ee, dd"')
+    length = Float.T(
+        default=1.0 * km,
+        help='Length of source rectangle [m].')
+    width = Float.T(
+        default=1.0 * km,
+        help='Width of source rectangle [m].')
+    torigin = Float.T(
+        default=0.0,
+        help='Origin time [days]')
+    idx = String.T(
+        default='nn',
+        help='Axis index for opening direction; "nn", "ee", or "dd"')
 
-    def to_rfs(self):
-        rf = -0.25
+    def to_rfs(self, nullification):
 
         cmpd = []
         for comp, mt in MTIso.items():
@@ -647,7 +699,7 @@ class PsCmpTensileSF(Location, gf.seismosizer.Cloneable):
 
             if comp != self.idx:
                 params = copy.deepcopy(mt)
-                params['opening'] *= rf
+                params['opening'] *= nullification
 
             kwargs = copy.deepcopy(self.__dict__)
             kwargs.update(params)
@@ -660,11 +712,25 @@ class PsCmpTensileSF(Location, gf.seismosizer.Cloneable):
 
 class PsCmpShearSF(Location, gf.seismosizer.Cloneable):
 
-    length = Float.T(default=1.0 * km)
-    width = Float.T(default=1.0 * km)
-    torigin = Float.T(default=0.0)
-    idx = String.T(default='ne',
-                   help='Axis Index for shear direction; "ne, nd, ed"')
+    '''
+    Shear fault source model component.
+
+    Warning: Mixed standard [m] / non-standard [days] units are used in this
+    class. Please read the documentation carefully.
+    '''
+
+    length = Float.T(
+        default=1.0 * km,
+        help='Length of source rectangle [m].')
+    width = Float.T(
+        default=1.0 * km,
+        help='Width of source rectangle [m].')
+    torigin = Float.T(
+        default=0.0,
+        help='Origin time [days]')
+    idx = String.T(
+        default='ne',
+        help='Axis index for opening direction; "ne", "nd", or "ed"')
 
     def to_rfs(self):
         kwargs = copy.deepcopy(self.__dict__)
@@ -678,21 +744,31 @@ class PsCmpMomentTensor(Location, gf.seismosizer.Cloneable):
     '''
     Mapping of Moment Tensor components to rectangular faults.
     Only one component at a time valid! NED coordinate system!
-    '''
-    length = Float.T(default=1.0 * km)
-    width = Float.T(default=1.0 * km)
-    torigin = Float.T(default=0.0)
-    idx = String.T(default='nn',
-                   help='Axis Index for MT component;'
-                        '"nn, ee, dd, ne, nd, ed"')
 
-    def to_rfs(self):
+    Warning: Mixed standard [m] / non-standard [days] units are used in this
+    class. Please read the documentation carefully.
+    '''
+    length = Float.T(
+        default=1.0 * km,
+        help='Length of source rectangle [m].')
+    width = Float.T(
+        default=1.0 * km,
+        help='Width of source rectangle [m].')
+    torigin = Float.T(
+        default=0.0,
+        help='Origin time [days]')
+    idx = String.T(
+        default='nn',
+        help='Axis index for MT component; '
+             '"nn", "ee", "dd", "ne", "nd", or "ed".')
+
+    def to_rfs(self, nullification=-0.25):
         kwargs = copy.deepcopy(self.__dict__)
         kwargs.pop('_latlon')
 
         if self.idx in MTIso:
             tsf = PsCmpTensileSF(**kwargs)
-            return tsf.to_rfs()
+            return tsf.to_rfs(nullification)
 
         elif self.idx in MTDev:
             ssf = PsCmpShearSF(**kwargs)
@@ -712,9 +788,9 @@ class PsCmpCoulombStressMasterFault(PsCmpCoulombStress):
     master_fault_strike = Float.T(default=300.)
     master_fault_dip = Float.T(default=15.)
     master_fault_rake = Float.T(default=90.)
-    sigma1 = Float.T(default=1.e6)
-    sigma2 = Float.T(default=-1.e6)
-    sigma3 = Float.T(default=0.0)
+    sigma1 = Float.T(default=1.e6, help='[Pa]')
+    sigma2 = Float.T(default=-1.e6, help='[Pa]')
+    sigma3 = Float.T(default=0.0, help='[Pa]')
 
     def string_for_config(self):
         return '%(friction)15e %(skempton_ratio)15e %(master_fault_strike)15f'\
@@ -725,7 +801,12 @@ class PsCmpCoulombStressMasterFault(PsCmpCoulombStress):
 class PsCmpSnapshots(Object):
     '''
     Snapshot time series definition.
+
+    Note: attributes in this class use non-standard units [days] to be
+    consistent with PSCMP text file input. Please read the documentation
+    carefully.
     '''
+
     tmin = Float.T(
         default=0.0,
         help='Time [days] after source time to start temporal sample'
@@ -760,7 +841,8 @@ class PsCmpConfig(Object):
              ' spacing of the GF database. This factor is applied to the'
              ' relationship in i.e. fault length & width = f * dx.')
 
-    snapshots = PsCmpSnapshots.T(PsCmpSnapshots.D())
+    snapshots = PsCmpSnapshots.T(
+        optional=True)
 
     rectangular_source_patches = List.T(PsCmpRectangularSource.T())
 
@@ -1112,12 +1194,22 @@ class PsCmpConfigFull(PsCmpConfig):
 class PsGrnPsCmpConfig(Object):
     '''
     Combined config Object of PsGrn and PsCmp.
-    '''
-    psgrn_config = PsGrnConfig.T(PsGrnConfig.D())
-    pscmp_config = PsCmpConfig.T(PsCmpConfig.D())
 
+    Note: attributes in this class use non-standard units [days] to be
+    consistent with PSCMP text file input. Please read the documentation
+    carefully.
+    '''
+    tmin_days = Float.T(
+        default=0.,
+        help='Min. time in days')
+    tmax_days = Float.T(
+        default=1.,
+        help='Max. time in days')
     gf_outdir = String.T(default='psgrn_functions')
 
+    psgrn_config = PsGrnConfig.T(default=PsGrnConfig.D())
+    pscmp_config = PsCmpConfig.T(default=PsCmpConfig.D())
+
 
 class PsCmpError(gf.store.StoreError):
     pass
@@ -1337,7 +1429,13 @@ class PsGrnCmpGFBuilder(gf.builder.Builder):
             lat=dummy_lat,
             lon=dummy_lon,
             points=points,
-            interpolation='nearest_neighbor')
+            interpolation='multilinear')
+
+        self.lambda_moduli = storeconf.get_lambda_moduli(
+            lat=dummy_lat,
+            lon=dummy_lon,
+            points=points,
+            interpolation='multilinear')
 
         if step == 0:
             block_size = (1, storeconf.nsource_depths, storeconf.ndistances)
@@ -1354,7 +1452,30 @@ class PsGrnCmpGFBuilder(gf.builder.Builder):
         baseconf = self.store.get_extra('psgrn_pscmp')
 
         cg = PsGrnConfigFull(**baseconf.psgrn_config.items())
+        if cg.n_time_samples is None or cg.max_time is None:
+            deltat_days = 1. / storeconf.sample_rate / day
+
+            cg.n_time_samples = int(baseconf.tmax_days // deltat_days)
+            cg.max_time = baseconf.tmax_days
+        else:
+            warnings.warn(
+                'PsGrnConfig is defining n_times_samples and max_time,'
+                ' this is replaced by PsGrnPsCmpConfig tmin and tmax.',
+                FutureWarning)
+
         cc = PsCmpConfigFull(**baseconf.pscmp_config.items())
+        if cc.snapshots is None:
+            deltat_days = 1. / storeconf.sample_rate / day
+
+            cc.snapshots = PsCmpSnapshots(
+                tmin=baseconf.tmin_days,
+                tmax=baseconf.tmax_days,
+                deltatdays=deltat_days)
+        else:
+            warnings.warn(
+                'PsCmpConfig is defining snapshots,'
+                ' this is replaced by PsGrnPsCmpConfig tmin and tmax.',
+                FutureWarning)
 
         cg.earthmodel_1d = storeconf.earthmodel_1d
 
@@ -1377,6 +1498,7 @@ class PsGrnCmpGFBuilder(gf.builder.Builder):
             (sz, firstx), (sz, lastx), (ns, nx) = \
                 self.get_block_extents(iblock)
             mu = self.shear_moduli[iblock]
+            lame_lambda = self.lambda_moduli[iblock]
 
             rz = self.store.config.receiver_depth
         else:
@@ -1394,10 +1516,20 @@ class PsGrnCmpGFBuilder(gf.builder.Builder):
             (self.step + 1, self.nsteps, iblock + 1, self.nblocks))
 
         if self.step == 0:
+            if cg.gf_depth_spacing is None:
+                gf_depth_spacing = fc.source_depth_delta
+            else:
+                gf_depth_spacing = cg.gf_depth_spacing * km
+
             n_steps_depth = int((fc.source_depth_max - fc.source_depth_min) /
-                                (cg.gf_depth_spacing * km)) + 1
+                                gf_depth_spacing) + 1
+
+            if cg.gf_distance_spacing is None:
+                gf_distance_spacing = fc.distance_delta
+            else:
+                gf_distance_spacing = cg.gf_distance_spacing * km
             n_steps_distance = int((fc.distance_max - fc.distance_min) /
-                                   (cg.gf_distance_spacing * km)) + 1
+                                   gf_distance_spacing) + 1
 
             cg.depth_grid = PsGrnSpatialSampling(
                 n_steps=n_steps_depth,
@@ -1433,11 +1565,13 @@ class PsGrnCmpGFBuilder(gf.builder.Builder):
             mtsize = float(dx * cc.rectangular_fault_size_factor)
 
             Ai = 1. / num.power(mtsize, 2)
+            nullf, sf = get_iso_scaling_factors(mu=mu, lame_lambda=lame_lambda)
+
             mui = 1. / mu
 
             gfmapping = [
                 (('nn',),
-                 {'un': (0, 0.4 * Ai * mui), 'ud': (5, 0.4 * Ai * mui)}),
+                 {'un': (0, Ai * sf), 'ud': (5, Ai * sf)}),
                 (('ne',),
                  {'ue': (3, 1 * Ai * mui)}),
                 (('nd',),
@@ -1445,9 +1579,9 @@ class PsGrnCmpGFBuilder(gf.builder.Builder):
                 (('ed',),
                  {'ue': (4, 1 * Ai * mui)}),
                 (('dd',),
-                 {'un': (2, 0.4 * Ai * mui), 'ud': (7, 0.4 * Ai * mui)}),
+                 {'un': (2, Ai * sf), 'ud': (7, Ai * sf)}),
                 (('ee',),
-                 {'un': (8, 0.4 * Ai * mui), 'ud': (9, 0.4 * Ai * mui)}),
+                 {'un': (8, Ai * sf), 'ud': (9, Ai * sf)}),
                 ]
 
             for mt, gfmap in gfmapping:
@@ -1461,7 +1595,8 @@ class PsGrnCmpGFBuilder(gf.builder.Builder):
                         length=mtsize,
                         idx=idx)
 
-                    cc.rectangular_source_patches.extend(pmt.to_rfs())
+                    cc.rectangular_source_patches.extend(
+                        pmt.to_rfs(nullf))
 
                 runner.run(cc)
 
