@@ -229,8 +229,9 @@ class QSeisConfigFull(QSeisConfig):
 
     def string_for_config(self):
 
-        def aggregate(l):
-            return len(l), '\n'.join([''] + [x.string_for_config() for x in l])
+        def aggregate(xx):
+            return len(xx), '\n'.join(
+                [''] + [x.string_for_config() for x in xx])
 
         assert len(self.receiver_distances) > 0
         assert len(self.receiver_distances) == len(self.receiver_azimuths)
@@ -792,6 +793,9 @@ class QSeisGFBuilder(gf.builder.Builder):
         if self.tmp is not None:
             util.ensuredir(self.tmp)
 
+    def cleanup(self):
+        self.store.close()
+
     def work_block(self, index):
         if len(self.store.config.ns) == 2:
             (sz, firstx), (sz, lastx), (ns, nx) = \
@@ -994,7 +998,7 @@ class QSeisGFBuilder(gf.builder.Builder):
                     (index+1, self.nblocks))
 
 
-def init(store_dir, variant):
+def init(store_dir, variant, config_params=None):
     if variant is None:
         variant = '2006'
 
@@ -1017,8 +1021,7 @@ def init(store_dir, variant):
 
     store_id = os.path.basename(os.path.realpath(store_dir))
 
-    config = gf.meta.ConfigTypeA(
-
+    d = dict(
         id=store_id,
         ncomponents=10,
         sample_rate=0.2,
@@ -1051,6 +1054,11 @@ def init(store_dir, variant):
                 id='s',
                 definition='!s')])
 
+    if config_params is not None:
+        d.update(config_params)
+
+    config = gf.meta.ConfigTypeA(**d)
+
     config.validate()
     return gf.store.Store.create_editables(
         store_dir, config=config, extra={'qseis': qseis})
