@@ -191,8 +191,9 @@ class QSeisSConfigFull(QSeisSConfig):
 
     def string_for_config(self):
 
-        def aggregate(l):
-            return len(l), '\n'.join([''] + [x.string_for_config() for x in l])
+        def aggregate(xx):
+            return len(xx), '\n'.join(
+                [''] + [x.string_for_config() for x in xx])
 
         assert self.earthmodel_1d is not None
         assert self.slowness_window is not None or self.calc_slowness_window,\
@@ -398,8 +399,9 @@ class QSeisRConfigFull(QSeisRConfig):
 
     def string_for_config(self):
 
-        def aggregate(l):
-            return len(l), '\n'.join([''] + [x.string_for_config() for x in l])
+        def aggregate(xx):
+            return len(xx), '\n'.join(
+                [''] + [x.string_for_config() for x in xx])
 
         assert self.earthmodel_receiver_1d is not None
 
@@ -876,6 +878,9 @@ class QSeis2dGFBuilder(gf.builder.Builder):
 
         util.ensuredir(baseconf.gf_directory)
 
+    def cleanup(self):
+        self.store.close()
+
     def work_block(self, iblock):
         if len(self.store.config.ns) == 2:
             (sz, firstx), (sz, lastx), (ns, nx) = \
