@@ -31,22 +31,23 @@ program_bins = {
     'qssp.2010beta': 'fomosto_qssp2010beta',
     'qssp.2010': 'fomosto_qssp2010',
     'qssp.2017': 'fomosto_qssp2017',
+    'qssp.2020': 'fomosto_qssp2020',
     'qssp.ppeg2017': 'fomosto_qsspppeg2017',
 }
 
 
-def have_backend():
-    have_any = False
-    for cmd in [[exe] for exe in program_bins.values()]:
+def have_backend(version=None):
+    avail = set()
+    for version, exe in program_bins.items():
         try:
-            p = Popen(cmd, stdout=PIPE, stderr=PIPE, stdin=PIPE)
+            p = Popen([exe], stdout=PIPE, stderr=PIPE, stdin=PIPE)
             (stdout, stderr) = p.communicate()
-            have_any = True
+            avail.add(version)
 
         except OSError:
             pass
 
-    return have_any
+    return avail
 
 
 qssp_components = {
@@ -54,7 +55,8 @@ qssp_components = {
     2: 'ar at ap gr sd tt tp ur ut up vr vt vp'.split(),
     3: '_disp_e _disp_n _disp_z'.split(),
     4: '_gravitation_e _gravitation_n _gravitation_z '
-       '_acce_e _acce_n _acce_z'.split()
+       '_acce_e _acce_n _acce_z'.split(),
+    5: '_rota_e _rota_n _rota_z'.split()
 }
 
 
@@ -190,6 +192,7 @@ class QSSPConfigFull(QSSPConfig):
     receiver_depth = Float.T(default=0.0)
     sampling_interval = Float.T(default=5.0)
 
+    stored_quantity = String.T(default='displacement')
     output_filename = String.T(default='receivers')
     output_format = Int.T(default=1)
     output_time_window = Float.T(optional=True)
@@ -213,8 +216,11 @@ class QSSPConfigFull(QSSPConfig):
 
     @property
     def components(self):
-        if self.qssp_version == '2017':
-            fmt = 3
+        if self.qssp_version in ('2017', '2020'):
+            if self.stored_quantity == "rotation":
+                fmt = 5
+            else:
+                fmt = 3
         elif self.qssp_version == 'ppeg2017':
             fmt = 4
         else:
@@ -223,7 +229,7 @@ class QSSPConfigFull(QSSPConfig):
         return qssp_components[fmt]
 
     def get_output_filenames(self, rundir):
-        if self.qssp_version in ('2017', 'ppeg2017'):
+        if self.qssp_version in ('2017', '2020', 'ppeg2017'):
             return [
                 pjoin(rundir, self.output_filename + c + '.dat')
                 for c in self.components]
@@ -237,8 +243,8 @@ class QSSPConfigFull(QSSPConfig):
 
     def string_for_config(self):
 
-        def aggregate(l):
-            return len(l), '\n'.join(x.string_for_config() for x in l)
+        def aggregate(xx):
+            return len(xx), '\n'.join(x.string_for_config() for x in xx)
 
         assert len(self.greens_functions) > 0
         assert len(self.sources) > 0
@@ -263,6 +269,12 @@ class QSSPConfigFull(QSSPConfig):
         model_str, nlines = cake_model_to_config(self.earthmodel_1d)
         d['n_model_lines'] = nlines
         d['model_lines'] = model_str
+        if self.stored_quantity == "rotation":
+            d['output_rotation'] = 1
+            d['output_displacement'] = 0
+        else:
+            d['output_displacement'] = 1
+            d['output_rotation'] = 0
 
         if len(self.sources) == 0 or isinstance(self.sources[0], QSSPSourceMT):
             d['point_source_type'] = 1
@@ -291,7 +303,7 @@ class QSSPConfigFull(QSSPConfig):
                 self.lowpass_order,
                 self.lowpass_corner)
 
-        elif self.qssp_version in ('2010', '2017', 'ppeg2017'):
+        elif self.qssp_version in ('2010', '2017', '2020', 'ppeg2017'):
             d['scutoff_doc'] = '''
 #    (SH waves), minimum and maximum cutoff harmonic degrees
 #    Note: if the near-field static displacement is desired, the minimum
@@ -317,7 +329,7 @@ class QSSPConfigFull(QSSPConfig):
                 self.bandpass_order,
                 self.bandpass_corner_low,
                 self.bandpass_corner_high)
-        if self.qssp_version in ('2017', 'ppeg2017'):
+        if self.qssp_version in ('2017', '2020', 'ppeg2017'):
             template = '''# autogenerated QSSP input by qssp.py
 #
 # This is the input file of FORTRAN77 program "qssp2017" for calculating
@@ -454,7 +466,8 @@ class QSSPConfigFull(QSSPConfig):
 # disp | velo | acce | strain | strain_rate | stress | stress_rate | rotation | rot_rate | gravitation | gravity
 #---------------------------------------------------------------------------------------------------------------
 # 1      1      1      1        1             1        1             1          1          1             1
-  1      0      1      0        0             0        0             0          0          1             0
+  %(output_displacement)i      0      0      0        0             0        0             %(output_rotation)i          0          0             0
+
   '%(output_filename)s'
   %(output_time_window)e
   %(sfilter)s
@@ -773,18 +786,34 @@ class QSSPGFBuilder(gf.builder.Builder):
 
         self.store = gf.store.Store(store_dir, 'w')
         baseconf = self.store.get_extra('qssp')
+        if baseconf.qssp_version in ('2017', '2020'):
+            if self.store.config.stored_quantity == "rotation":
+                self.gfmapping = [
+                    (MomentTensor(m=symmat6(1, 0, 0, 1, 0, 0)),
+                     {'_rota_n': (0, -1), '_rota_e': (3, -1),
+                      '_rota_z': (5, -1)}),
+                    (MomentTensor(m=symmat6(0, 0, 0, 0, 1, 1)),
+                     {'_rota_n': (1, -1), '_rota_e': (4, -1),
+                      '_rota_z': (6, -1)}),
+                    (MomentTensor(m=symmat6(0, 0, 1, 0, 0, 0)),
+                     {'_rota_n': (2, -1), '_rota_z': (7, -1)}),
+                    (MomentTensor(m=symmat6(0, 1, 0, 0, 0, 0)),
+                     {'_rota_n': (8, -1), '_rota_z': (9, -1)}),
+                ]
+            else:
+                self.gfmapping = [
+                    (MomentTensor(m=symmat6(1, 0, 0, 1, 0, 0)),
+                     {'_disp_n': (0, -1), '_disp_e': (3, -1),
+                      '_disp_z': (5, -1)}),
+                    (MomentTensor(m=symmat6(0, 0, 0, 0, 1, 1)),
+                     {'_disp_n': (1, -1), '_disp_e': (4, -1),
+                      '_disp_z': (6, -1)}),
+                    (MomentTensor(m=symmat6(0, 0, 1, 0, 0, 0)),
+                     {'_disp_n': (2, -1), '_disp_z': (7, -1)}),
+                    (MomentTensor(m=symmat6(0, 1, 0, 0, 0, 0)),
+                     {'_disp_n': (8, -1), '_disp_z': (9, -1)}),
+                ]
 
-        if baseconf.qssp_version == '2017':
-            self.gfmapping = [
-                (MomentTensor(m=symmat6(1, 0, 0, 1, 0, 0)),
-                 {'_disp_n': (0, -1), '_disp_e': (3, -1), '_disp_z': (5, -1)}),
-                (MomentTensor(m=symmat6(0, 0, 0, 0, 1, 1)),
-                 {'_disp_n': (1, -1), '_disp_e': (4, -1), '_disp_z': (6, -1)}),
-                (MomentTensor(m=symmat6(0, 0, 1, 0, 0, 0)),
-                 {'_disp_n': (2, -1), '_disp_z': (7, -1)}),
-                (MomentTensor(m=symmat6(0, 1, 0, 0, 0, 0)),
-                 {'_disp_n': (8, -1), '_disp_z': (9, -1)}),
-            ]
         elif baseconf.qssp_version == 'ppeg2017':
             self.gfmapping = [
                 (MomentTensor(m=symmat6(1, 0, 0, 1, 0, 0)),
@@ -824,7 +853,6 @@ class QSSPGFBuilder(gf.builder.Builder):
         conf.gf_directory = pjoin(store_dir, 'qssp_green')
         conf.earthmodel_1d = self.store.config.earthmodel_1d
         deltat = self.store.config.deltat
-
         if 'time_window' not in shared:
             d = self.store.make_timing_params(
                 conf.time_region[0], conf.time_region[1],
@@ -847,6 +875,9 @@ class QSSPGFBuilder(gf.builder.Builder):
 
         self.qssp_config = conf
 
+    def cleanup(self):
+        self.store.close()
+
     def work_block(self, iblock):
         if len(self.store.config.ns) == 2:
             (sz, firstx), (sz, lastx), (ns, nx) = \
@@ -879,8 +910,8 @@ class QSSPGFBuilder(gf.builder.Builder):
         runner = QSSPRunner(tmp=self.tmp)
 
         conf.receiver_depth = rz/km
+        conf.stored_quantity = self.store.config.stored_quantity
         conf.sampling_interval = 1.0 / self.gf_config.sample_rate
-
         dx = self.gf_config.distance_delta
 
         if self.step == 0:
@@ -1041,7 +1072,7 @@ class QSSPGFBuilder(gf.builder.Builder):
 km = 1000.
 
 
-def init(store_dir, variant):
+def init(store_dir, variant, config_params=None):
     if variant is None:
         variant = '2010beta'
 
@@ -1086,7 +1117,7 @@ def init(store_dir, variant):
     else:
         sample_rate = 0.2
 
-    config = gf.meta.ConfigTypeA(
+    d = dict(
         id=store_id,
         ncomponents=10,
         component_scheme='elastic10',
@@ -1121,6 +1152,11 @@ def init(store_dir, variant):
                 id='s',
                 definition='!s')])
 
+    if config_params is not None:
+        d.update(config_params)
+
+    config = gf.meta.ConfigTypeA(**d)
+
     config.validate()
     return gf.store.Store.create_editables(
         store_dir,
@@ -1139,3 +1175,14 @@ def build(
     return QSSPGFBuilder.build(
         store_dir, force=force, nworkers=nworkers, continue_=continue_,
         step=step, iblock=iblock)
+
+
+def get_conf(
+        store_dir,
+        force=False,
+        nworkers=None,
+        continue_=False,
+        step=None,
+        iblock=None):
+
+    return QSSPGFBuilder.get_conf()
