@@ -391,7 +391,7 @@ timing_regex_old = re.compile(
 
 timing_regex = re.compile(
     r'^((first|last)?\{(' + _ppat + r'(\|' + _ppat + r')*)\}|(' +
-    _ppat + r'))?(' + _fpat + 'S?)?$')
+    _ppat + r'))?(' + _fpat + '(S|%)?)?$')
 
 
 class PhaseSelect(StringChoice):
@@ -412,11 +412,11 @@ class Timing(SObject):
 
     Timings can be instantiated from a simple string defintion i.e. with
     ``Timing(str)`` where ``str`` is something like
-    ``'SELECT{PHASE_DEFS}[+-]OFFSET[S]'`` where ``'SELECT'`` is ``'first'``,
+    ``'SELECT{PHASE_DEFS}[+-]OFFSET[S|%]'`` where ``'SELECT'`` is ``'first'``,
     ``'last'`` or empty, ``'PHASE_DEFS'`` is a ``'|'``-separated list of phase
-    definitions, and ``'OFFSET'`` is the time offset in seconds. If the an
-    ``'S'`` is appended to ``'OFFSET'``, it is interpreted as a surface
-    slowness in [s/km].
+    definitions, and ``'OFFSET'`` is the time offset in seconds. If a ``'%'``
+    is appended, it is interpreted as percent. If the an ``'S'`` is appended
+    to ``'OFFSET'``, it is interpreted as a surface slowness in [s/km].
 
     Phase definitions can be specified in either of the following ways:
 
@@ -432,6 +432,10 @@ class Timing(SObject):
     * ``'100'`` : absolute time; 100 s
     * ``'{stored:P}-100'`` : 100 s before arrival of P phase according to
       stored travel time table named ``'P'``
+    * ``'{stored:P}-5.1S'`` : 10% before arrival of P phase according to
+      stored travel time table named ``'P'``
+    * ``'{stored:P}-10%'`` : 10% before arrival of P phase according to
+      stored travel time table named ``'P'``
     * ``'{stored:A|stored:B}'`` : time instant of phase arrival A, or B if A is
       undefined for a given geometry
     * ``'first{stored:A|stored:B}'`` : as above, but the earlier arrival of A
@@ -444,11 +448,14 @@ class Timing(SObject):
     def __init__(self, s=None, **kwargs):
 
         if s is not None:
-            offset_is_slowness = False
+            offset_is = None
             s = re.sub(r'\s+', '', s)
             try:
                 offset = float(s.rstrip('S'))
-                offset_is_slowness = s.endswith('S')
+
+                if s.endswith('S'):
+                    offset_is = 'slowness'
+
                 phase_defs = []
                 select = ''
 
@@ -468,8 +475,11 @@ class Timing(SObject):
                     offset = 0.0
                     soff = m.group(27)
                     if soff:
-                        offset = float(soff.rstrip('S'))
-                        offset_is_slowness = soff.endswith('S')
+                        offset = float(soff.rstrip('S%'))
+                        if soff.endswith('S'):
+                            offset_is = 'slowness'
+                        elif soff.endswith('%'):
+                            offset_is = 'percent'
 
                     matched = True
 
@@ -501,7 +511,7 @@ class Timing(SObject):
                 phase_defs=phase_defs,
                 select=select,
                 offset=offset,
-                offset_is_slowness=offset_is_slowness)
+                offset_is=offset_is)
 
         SObject.__init__(self, **kwargs)
 
@@ -519,14 +529,16 @@ class Timing(SObject):
 
         if self.offset != 0.0 or not self.phase_defs:
             s.append('%+g' % self.offset)
-            if self.offset_is_slowness:
+            if self.offset_is == 'slowness':
                 s.append('S')
+            elif self.offset_is == 'percent':
+                s.append('%')
 
         return ''.join(s)
 
     def evaluate(self, get_phase, args):
         try:
-            if self.offset_is_slowness and self.offset != 0.0:
+            if self.offset_is == 'slowness' and self.offset != 0.0:
                 phase_offset = get_phase(
                     'vel_surface:%g' % (1.0/self.offset))
                 offset = phase_offset(args)
@@ -537,7 +549,12 @@ class Timing(SObject):
                 phases = [
                     get_phase(phase_def) for phase_def in self.phase_defs]
                 times = [phase(args) for phase in phases]
-                times = [t+offset for t in times if t is not None]
+                if self.offset_is == 'percent':
+                    times = [t*(1.+offset/100.)
+                             for t in times if t is not None]
+                else:
+                    times = [t+offset for t in times if t is not None]
+
                 if not times:
                     return None
                 elif self.select == 'first':
@@ -554,7 +571,7 @@ class Timing(SObject):
 
     phase_defs = List.T(String.T())
     offset = Float.T(default=0.0)
-    offset_is_slowness = Bool.T(default=False)
+    offset_is = String.T(optional=True)
     select = PhaseSelect.T(
         default='',
         help=('Can be either ``\'%s\'``, ``\'%s\'``, or ``\'%s\'``. ' %
@@ -1717,6 +1734,52 @@ class Config(Object):
                                           parameter='shear_moduli',
                                           interpolation=interpolation)
 
+    def get_lambda_moduli(self, lat, lon, points,
+                          interpolation=None):
+        '''
+        Get lambda moduli at given points from contained velocity model.
+
+        :param lat: surface origin for coordinate system of ``points``
+        :param points: NumPy array of shape ``(N, 3)``, where each row is
+            a point ``(north, east, depth)``, relative to origin at
+            ``(lat, lon)``
+        :param interpolation: interpolation method. Choose from
+            ``('nearest_neighbor', 'multilinear')``
+        :returns: NumPy array of length N with extracted shear moduli at each
+            point
+
+        The default implementation retrieves and interpolates the lambda moduli
+        from the contained 1D velocity profile.
+        '''
+        return self.get_material_property(lat, lon, points,
+                                          parameter='lambda_moduli',
+                                          interpolation=interpolation)
+
+    def get_bulk_moduli(self, lat, lon, points,
+                        interpolation=None):
+        '''
+        Get bulk moduli at given points from contained velocity model.
+
+        :param lat: surface origin for coordinate system of ``points``
+        :param points: NumPy array of shape ``(N, 3)``, where each row is
+            a point ``(north, east, depth)``, relative to origin at
+            ``(lat, lon)``
+        :param interpolation: interpolation method. Choose from
+            ``('nearest_neighbor', 'multilinear')``
+        :returns: NumPy array of length N with extracted shear moduli at each
+            point
+
+        The default implementation retrieves and interpolates the lambda moduli
+        from the contained 1D velocity profile.
+        '''
+        lambda_moduli = self.get_material_property(
+            lat, lon, points, parameter='lambda_moduli',
+            interpolation=interpolation)
+        shear_moduli = self.get_material_property(
+            lat, lon, points, parameter='shear_moduli',
+            interpolation=interpolation)
+        return lambda_moduli + (2 / 3) * shear_moduli
+
     def get_vs(self, lat, lon, points, interpolation=None):
         '''
         Get Vs at given points from contained velocity model.
@@ -1814,6 +1877,22 @@ class Config(Object):
                 store_depth_profile, z_profile, rho_profile)
 
             profile = num.power(store_vs_profile, 2) * store_rho_profile
+
+        elif parameter == 'lambda_moduli':
+            vs_profile = earthmod.profile('vs')
+            vp_profile = earthmod.profile('vp')
+            rho_profile = earthmod.profile('rho')
+
+            store_vs_profile = num.interp(
+                store_depth_profile, z_profile, vs_profile)
+            store_vp_profile = num.interp(
+                store_depth_profile, z_profile, vp_profile)
+            store_rho_profile = num.interp(
+                store_depth_profile, z_profile, rho_profile)
+
+            profile = store_rho_profile * (
+                num.power(store_vp_profile, 2) -
+                num.power(store_vs_profile, 2) * 2)
         else:
             raise TypeError(
                 'parameter %s not available' % parameter)
@@ -1834,8 +1913,9 @@ class Config(Object):
             raise OutOfBounds()
 
     def is_static(self):
-        if self.modelling_code_id in ('psgrn_pscmp', 'poel'):
-            return True
+        for code in ('psgrn_pscmp', 'poel'):
+            if self.modelling_code_id.startswith(code):
+                return True
         return False
 
     def is_dynamic(self):
