@@ -20,10 +20,13 @@ import asyncore
 import socket
 try:
     from http.server import SimpleHTTPRequestHandler as SHRH
+    from html import escape
 except ImportError:
     from SimpleHTTPServer import SimpleHTTPRequestHandler as SHRH
+    from cgi import escape
+
 import sys
-import html
+
 import json
 import cgi
 from io import BytesIO
@@ -38,10 +41,10 @@ import logging
 import matplotlib
 matplotlib.use('Agg')  # noqa
 
-import matplotlib.pyplot as plt
-from pyrocko.plot import cake_plot
-from pyrocko import gf, util
-from pyrocko.util import quote, unquote
+import matplotlib.pyplot as plt  # noqa
+from pyrocko.plot import cake_plot  # noqa
+from pyrocko import gf, util  # noqa
+from pyrocko.util import quote, unquote  # noqa
 
 try:
     newstr = unicode
@@ -136,7 +139,10 @@ class RequestHandler(asynchat.async_chat, SHRH):
 
     def prepare_POST(self):
         """Prepare to read the request body"""
-        bytesToRead = int(self.headers.getheader('Content-length'))
+        try:
+            bytesToRead = int(self.headers.getheader('Content-length'))
+        except AttributeError:
+            bytesToRead = int(self.headers['Content-length'])
         # set terminator to length (will read bytesToRead bytes)
         self.set_terminator(bytesToRead)
         self.incoming.clear()
@@ -174,8 +180,15 @@ class RequestHandler(asynchat.async_chat, SHRH):
     def do_POST(self):
         """Begins serving a POST request. The request data must be readable
         on a file-like object called self.rfile"""
-        ctype, pdict = cgi.parse_header(self.headers.getheader('content-type'))
-        length = int(self.headers.getheader('content-length'))
+        try:
+            data = cgi.parse_header(self.headers.getheader('content-type'))
+            length = int(self.headers.getheader('content-length', 0))
+        except AttributeError:
+            data = cgi.parse_header(self.headers.get('content-type'))
+            length = int(self.headers.get('content-length', 0))
+
+        ctype, pdict = data if data else (None, None)
+
         if ctype == 'multipart/form-data':
             self.body = cgi.parse_multipart(self.rfile, pdict)
         elif ctype == 'application/x-www-form-urlencoded':
@@ -234,7 +247,13 @@ class RequestHandler(asynchat.async_chat, SHRH):
             self.send_error(501, "Unsupported method (%s)" % self.command)
 
     def handle_error(self):
-        traceback.print_exc(sys.stderr)
+        try:
+            traceback.print_exc(sys.stderr)
+        except Exception:
+            logger.error(
+                'An error occurred and another one while printing the '
+                'traceback. Please debug me...')
+
         self.close()
 
     def writable(self):
@@ -338,7 +357,7 @@ class RequestHandler(asynchat.async_chat, SHRH):
 
         list.sort(key=lambda a: a.lower())
         f = BytesIO()
-        displaypath = html.escape(unquote(self.path))
+        displaypath = escape(unquote(self.path))
         f.write(enc('<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">'))
         f.write(enc("<html>\n<title>Directory listing for %s</title>\n"
                 % displaypath))
@@ -357,7 +376,7 @@ class RequestHandler(asynchat.async_chat, SHRH):
                 # Note: a link to a directory displays with @ and links with /
             f.write(enc('<li><a href="%s">%s</a>\n' %
                         (quote(linkname),
-                         html.escape(displayname))))
+                         escape(displayname))))
         f.write(enc("</ul>\n<hr>\n</body>\n</html>\n"))
         length = f.tell()
         f.seek(0)
@@ -646,6 +665,11 @@ class SeismosizerHandler(RequestHandler):
             self.end_headers()
             return
 
+        if store.config.earthmodel_1d is None:
+            self.send_error(404)
+            self.end_headers()
+            return
+
         cake_plot.my_model_plot(store.config.earthmodel_1d, axes=axes)
 
         f = BytesIO()
