@@ -456,6 +456,7 @@ class MomentTensor(Object):
 
     :param m: NumPy matrix in north-east-down convention
     :param m_up_south_east: NumPy matrix in up-south-east convention
+    :param m_east_north_up: NumPy matrix in east-north-up convention
     :param strike,dip,rake: fault plane angles in [degrees]
     :param scalar_moment: scalar moment in [Nm]
     :param magnitude: moment magnitude Mw
@@ -494,6 +495,8 @@ class MomentTensor(Object):
         [[0., 0., -1.], [0., -1., 0.], [-1., 0., 0.]], dtype=num.float)
     _to_up_south_east = num.matrix(
         [[0., 0., -1.], [-1., 0., 0.], [0., 1., 0.]], dtype=num.float).T
+    _to_east_north_up = num.matrix(
+        [[0., 1., 0.], [1., 0., 0.], [0., 0., -1.]], dtype=num.float)
     _m_unrot = num.matrix(
         [[0., 0., -1.], [0., 0., 0.], [-1., 0., 0.]], dtype=num.float)
 
@@ -545,7 +548,7 @@ class MomentTensor(Object):
         return MomentTensor(m=m)
 
     def __init__(
-            self, m=None, m_up_south_east=None,
+            self, m=None, m_up_south_east=None, m_east_north_up=None,
             strike=0., dip=0., rake=0., scalar_moment=1.,
             mnn=None, mee=None, mdd=None, mne=None, mnd=None, med=None,
             strike1=None, dip1=None, rake1=None,
@@ -565,6 +568,10 @@ class MomentTensor(Object):
             m = self._to_up_south_east * m_up_south_east * \
                 self._to_up_south_east.T
 
+        if m_east_north_up is not None:
+            m = self._to_east_north_up * m_east_north_up * \
+                self._to_east_north_up.T
+
         if m is None:
             if any(x is not None for x in (
                     strike1, dip1, rake1, strike2, dip2, rake2)):
@@ -777,7 +784,7 @@ class MomentTensor(Object):
 
         .. math::
 
-            M0 = \\frac{1}{\\sqrt{2}}\\sqrt{\\sum_{i,j} |M_{ij}|^2}
+            M_0 = \\frac{1}{\\sqrt{2}}\\sqrt{\\sum_{i,j} |M_{ij}|^2}
 
         The scalar moment is calculated based on the Euclidean (Frobenius) norm
         (Silver and Jordan, 1982). The scalar moment returned by this function
